package CoBlog::Controller::Admin::Author;

use strict;
use warnings;
use base 'Catalyst::Controller';
use FormElementContainer;

=head1 NAME

CoBlog::Controller::Admin::Author - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut


=head2 index 

=cut

sub index : Private {
    my ( $self, $c ) = @_;

    $c->forward('list');
}

=head2 list

List of all author

=cut

sub list : Local {
    my ( $self, $c, $return_count ) = @_;
    
    $c->stash->{authors} = [ sort { $a->username cmp $b->username } $c->model('CoBlogDB::Author')->all ];
    $c->stash->{template}='admin/author/list.tt2';
}

=head2 make_author_widget

Build an HTML::Widget form for author creation and updates

=cut

sub make_author_widget {
    my ($self, $c) = @_;

    # Create an HTML::Widget to build the form
    my $w = $c->widget('author_form')->method('post');

    # Use custom class to render each element in the form    
    $w->element_container_class('FormElementContainer');
                
    # Create the form feilds
    $w->element('Hidden',    'id'       )->value("");
    $w->element('Textfield', 'username'  )->label('Username');
    $w->element('Textfield', 'email' )->label('Email');
    $w->element('Textfield', 'firstname' )->label('Firstname');
    $w->element('Textfield', 'lastname' )->label('Lastname');
    $w->element('Password', 'password' )->label('Password');
    $w->element('Password', 'verify_password' )->label('Verify password');
    $w->element('Submit',    'submit' )->value('submit');

    # Set constraints
    $w->constraint(All     => qw/username email password verify_password/)
        ->message('Required. ');
    $w->constraint(Length  => qw/username email/)->min(2)
        ->message('Must be minimum 2 characters. ');
    $w->constraint(Length  => qw/password verify_password/)->min(5)
        ->message('Must be minimum 5 characters. ');
    $w->constraint(Email  => qw/email/)
        ->message('Please use a valid email address. ');
    $w->constraint(String  => qw/username password verify_password/)
        ->message('Please use only normal characters. No spaces allowed.');
    $w->constraint(Equal  => qw/password verify_password/)
        ->message('Your password does not equal the verify password.');

    # Set filters
    for my $column (qw/username email firstname lastname/) {
        $w->filter( HTMLEscape => $column );
        $w->filter( TrimEdges  => $column );
    }

    # Return the widget    
    return $w;
}

=head2 edit

Build an HTML::Widget form for author creation and updates

=cut

sub edit : Local {
    my ($self, $c, $author_id ) = @_;

    # Create the widget and set the action for the form
    my $w = $self->make_author_widget($c);
    $w->action($c->uri_for('create_do'));

    if ( my $author=$c->model('CoBlogDB::Author')->find({ id => $author_id}) ) {
        $author->fill_widget( $w );

        # UTF8 fill_widget kludge (fill_widget seems to not adhere to utf8 state)
        # Doesn't work. This must be a bug somewhere in DBIx::Class, Catalyst or HTML::Widget
#        use Encode ();
#        (my $firstname_element)=$w->get_elements( name => 'firstname' );
#        (my $lastname_element)=$w->get_elements( name => 'lastname' );
#        $c->log->debug("Firstname W: " . $firstname_element->value() );
#        $c->log->debug("Firstname D: " . $author->firstname );
#        $c->log->debug("Lastname W: " . $lastname_element->value() );
#        $c->log->debug("Lastname D: " . $author->lastname );
#        $firstname_element->value( Encode::encode('utf-8', $author->firstname) );
#        $lastname_element->value( Encode::encode('ISO-8859-1', $author->lastname) );
        
        # Write form to stash variable for use in template
        $c->stash->{widget_result} = $w->result;

        # Set the template
        $c->stash->{template} = 'admin/author/create.tt2';
    }
    else {
        # Notify user and return to index page
        $c->stash->{error_msg} = 'Author could not be found!';
        $c->forward('index');
    }
    
}


=head2 create

Build an HTML::Widget form for author creation and updates

=cut

sub create : Local {
    my ($self, $c) = @_;

    # Create the widget and set the action for the form
    my $w = $self->make_author_widget($c);
    $w->action($c->uri_for('create_do'));

    # Write form to stash variable for use in template
    $c->stash->{widget_result} = $w->result;

    # Set the template
    $c->stash->{template} = 'admin/author/create.tt2';
}

=head2 create_do

Build an HTML::Widget form for author creation and updates

=cut

sub create_do : Local {
    my ($self, $c) = @_;

    # Create the widget and set the action for the form
    my $w = $self->make_author_widget($c);
    $w->action($c->uri_for('create_do'));

    # Validate the form parameters
    my $result = $w->process($c->req);

    # Write form (including validation error messages) to
    # stash variable for use in template
    $c->stash->{widget_result} = $result;

    # Were their validation errors?
    if ($result->has_errors) {
        # Warn the user at the top of the form that there were errors.
        # Note that there will also be per-field feedback on
        # validation errors because of '$w->process($c->req)' above.
        $c->stash->{error_msg} = 'Errors in form. Please verify your information!';

        # Set the template
        $c->stash->{template} = 'admin/author/create.tt2';

    } else {

        # Check that username/email is not already taken
        if ( !$result->params->{'id'} and $c->model('CoBlogDB::Author')->find({ username => $result->params->{'username'} }) ) {
            $c->stash->{error_msg} = 'Username is not available! Please choose another one.';
            $c->stash->{template} = 'admin/author/create.tt2';
        }
        elsif ( !$result->params->{'id'} and my $old_author = $c->model('CoBlogDB::Author')->find({ email => $result->params->{'email'} }) ) {
            $c->stash->{error_msg} = 'Another user is already registered with this email address!<br /> Please login using the username <strong>' . $old_author->username . '</strong> to access the account.';
            $c->stash->{template} = 'admin/author/create.tt2';
        }
        else {
            my $author = $result->params->{'id'} ? $c->model('CoBlogDB::Author')->find({ id => $result->params->{'id'} })
                                                 : $c->model('CoBlogDB::Author')->new({});
            $author->populate_from_widget($result);

            # Set a status message for the user
            $c->flash->{status_msg} = $result->params->{'id'} ? 'Author information updated.' : 'Author created.';
            
            # Redisplay an empty form for another
            $c->stash->{widget_result} = $w->result;

            # Redirect back to list
            $c->response->redirect($c->uri_for('list'));
        }

    }

}

=head2 delete 

Delete an author
    
=cut

sub delete : Local {
    # $id = primary key of book to delete
    my ($self, $c, $id) = @_;

    # Search for the author and then delete it (using delete_all will also remove all articles this author owns).
    $c->model('CoBlogDB::Author')->search({id => $id})->delete;

    # Set a status message to be displayed at the top of the view
    $c->flash->{status_msg} = "Author deleted.";

    $c->response->redirect($c->uri_for('/admin/author/list'));
}

=head1 AUTHOR

Robin Smidsrød,,,

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
