package CoBlog::Controller::Article;

use strict;
use warnings;
use base 'Catalyst::Controller';

=head1 NAME

CoBlog::Controller::Article - Catalyst Controller

=head1 DESCRIPTION

Catalyst Controller.

=head1 METHODS

=cut


=head2 index

Default action

=cut

sub index : Private {
    my ( $self, $c ) = @_;
    
    $c->forward('list');
}

=head2 list

Frontpage view

=cut

sub list : Local {
    my ( $self, $c, $return_count ) = @_;
    
    # Fetch latest articles
    my @all_articles=sort { $b->updated cmp $a->updated } $c->model('CoBlogDB::Article')->all;

    $return_count ||=5;
    unless ( $return_count=~/^all$/i ) {
        splice @all_articles, $return_count;
    }
    $c->stash->{articles} = \@all_articles;
    
    # Set default front page
    $c->stash->{template}='article/list.tt2';
}


=head2 read

Single article view

=cut

sub read : Local {
    my ( $self, $c, $article_id ) = @_;
    $c->stash->{article} = $c->model('CoBlogDB::Article')->find({ id => $article_id });
    $c->stash->{template}='article/read.tt2';
}

=head2 search

Search view

=cut

sub search : Local {
    my ( $self, $c, $search, $return_count ) = @_;
    
    # Try to fetch from CGI unless set
    $search ||= $c->request->params->{q};
    $return_count ||= $c->request->params->{count};
    
    # Fetch articles that match search
    my @all_articles=$c->model('CoBlogDB::Article')->search( {}, {order_by => \'updated DESC', limit => 10} )
             ->search_literal('MATCH (title, content) AGAINST( ? )', $search) ;

    $return_count ||=5;
    unless ( $return_count=~/^all$/i ) {
        splice @all_articles, $return_count;
    }
    $c->stash->{articles} = \@all_articles;

    $c->stash->{is_search} = 1;
    $c->stash->{search_phrase}=$search;
    
    # Set default front page
    $c->stash->{template}='article/list.tt2';
}

=head2 suggest

Suggest Search view

=cut

sub suggest : Local {
    my ( $self, $c, $search ) = @_;
    
    # Try to fetch from CGI unless set
    $search ||= $c->request->params->{q};
    
    if ($search) {
        # Fetch articles that match search
        my @all_articles=$c->model('CoBlogDB::Article')->search( {}, {order_by => \'updated DESC', limit => 10} )
                 ->search_literal('MATCH (title, content) AGAINST( ? )', $search) ;

        $c->stash->{articles} = \@all_articles;
        
    }

    # Using the filename .txt.tt2 forces text/plain mimetype
    $c->stash->{template}='article/suggest.txt.tt2';

}

=head1 AUTHOR

Robin Smidsrød,,,

=head1 LICENSE

This library is free software, you can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

1;
